//+------------------------------------------------------------------+
//|                                                          RVI.mq4 |
//|                                                                  |
//+------------------------------------------------------------------+
#property  copyright ""
#property  link      ""

#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  Green
#property  indicator_color2  OrangeRed

//
//
//
//
//

extern string  TimeFrame       = "Current time frame";
extern int     RVIPeriod       = 10;
extern bool    alertsOn        = false;
extern bool    alertsOnCurrent = true;
extern bool    alertsMessage   = true;
extern bool    alertsSound     = false;
extern bool    alertsNotify    = false;
extern bool    alertsEmail     = false;

//
//
//
//
//

double RVIBuffer[];
double RVISignalBuffer[];
double buffer1[];
double buffer2[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
      SetIndexBuffer(0,RVIBuffer);
      SetIndexBuffer(1,RVISignalBuffer);
      SetIndexBuffer(2,buffer1);
      SetIndexBuffer(3,buffer2);
      SetIndexBuffer(4,trend);
   SetIndexDrawBegin(0,RVIPeriod+3);   
   SetIndexDrawBegin(1,RVIPeriod+7); 
   
   //
   //
   //
   //
   //
     
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+" RVI ("+RVIPeriod+")");
   SetIndexLabel(1,"RVI Signal");
   SetIndexLabel(0,"RVI");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { RVIBuffer[0] = limit+1; return(0); }
   
            if (timeFrame!=Period())
            {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,timeFrame,Time[i]);               
                      RVIBuffer[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RVIPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,0,y);
                      RVISignalBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RVIPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,1,y);
               }
               return(0);
            }

   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
      buffer1[i] = ((Close[i]-Open[i])+2*(Close[i+1]-Open[i+1])+2*(Close[i+2]-Open[i+2])+(Close[i+3]-Open[i+3]))/6;
      buffer2[i] = ((High[i]-Low[i])  +2*(High[i+1]-Low[i+1])  +2*(High[i+2]-Low[i+2])  +(High[i+3]-Low[i+3])  )/6;
      
      //
      //
      //
      //
      //
      
      double num   = 0.0; 
      double denom = 0.0;
         for(int k=0; k<RVIPeriod; k++)
         {
            num   += buffer1[i+k];
            denom += buffer2[i+k];
         }
         if(denom!=0.0)
               RVIBuffer[i] = num/denom;
         else  RVIBuffer[i] = 0.0;
         RVISignalBuffer[i] = (RVIBuffer[i]+2*RVIBuffer[i+1]+2*RVIBuffer[i+2]+RVIBuffer[i+3])/6;
         trend[i] = trend[i+1];
            if (RVIBuffer[i]>RVISignalBuffer[i]) trend[i] =  1;
            if (RVIBuffer[i]<RVISignalBuffer[i]) trend[i] = -1;
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("up");
      else  doAlert("down");       
   }
   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+  
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," RVI trend changed to ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," RVI " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," RVI "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

